### A Pluto.jl notebook ###
# v0.20.4

using Markdown
using InteractiveUtils

# This Pluto notebook uses @bind for interactivity. When running this notebook outside of Pluto, the following 'mock version' of @bind gives bound variables a default value (instead of an error).
macro bind(def, element)
    #! format: off
    quote
        local iv = try Base.loaded_modules[Base.PkgId(Base.UUID("6e696c72-6542-2067-7265-42206c756150"), "AbstractPlutoDingetjes")].Bonds.initial_value catch; b -> missing; end
        local el = $(esc(element))
        global $(esc(def)) = Core.applicable(Base.get, el) ? Base.get(el) : iv(el)
        el
    end
    #! format: on
end

# ╔═╡ 92c861a8-3027-4f39-a310-986aa47eea8f
begin
	using PlutoUI, Dates, SparseArrays , StatsBase, PlutoPlotly, PlotlyKaleido 
	using CSV ,  DataFrames, LinearAlgebra , TimeSeries, NLsolve, LaTeXStrings
	force_pluto_mathjax_local(true)
	PlotlyKaleido.start(;mathjax = true)
end

# ╔═╡ 576d5509-527d-4c48-9f2e-007fc977a0d8
html"""
<picture>
  <source srcset="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/negative/rgb_ibs_en_horizontal_negative.png" media="(prefers-color-scheme: dark)" width="240px!important" style="float:right;">
  <img src="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/main/rgb_ibs_en_horizontal_main.png" width="240px" style="float:right;">
</picture>
"""

# ╔═╡ 08782891-e0f5-4c25-9657-24212101724c
md"""
# Filters & Impulse Response Functions

**26 February 2025**
"""

# ╔═╡ 67198764-1378-4286-8779-8a05011bf084
md"""
##### Vivaldo Mendes
"""

# ╔═╡ 9c11028c-8ccd-4c17-a964-86f09f785ba0
md"""
---
"""

# ╔═╡ e8f80d8c-5b6d-449a-8151-7c6f057492ae
md"""
### Packages used in this notebook
"""

# ╔═╡ c76ef346-c56a-4d0d-a73b-8b81c3688cc4


# ╔═╡ 637413e8-c2d2-46d9-ac9c-9a57779bcc33
md"""
## 1. The Hodrick-Prescott filter
"""

# ╔═╡ 28eff862-eb6a-4546-bfc7-f334aced0e5e
md"""
### The data
"""

# ╔═╡ 11943aea-98c2-4dba-aa08-ab4241a06ffc
USdata = CSV.read("Data_US.csv", DataFrame)

# ╔═╡ 51a0bef0-7b98-4513-9b83-63b1c944d9a8
md"""
If we want to work with a particular period, we have to stipulate the _initial date_ and the _end date_ like this:

		my_new_data = USdata[initial date .<= USdata.Quarters .<=  end date, :]
		
"""

# ╔═╡ 5e99d909-c2f3-4880-9855-edc3f791370e
my_new_data = USdata[Date(1960,01,01) .<= USdata.Quarters .<= Date(1961,10,01), :] 			# Dates passed as integers
#my_new_data = USdata[Date("1960-01-01") .<= USdata.Quarters .<= Date("1961-04-01"), :] 	# Dates passed as strings

# ╔═╡ 93a211cf-5ad7-4fdc-a2fb-6981e61a0e3a
begin
	pp1 = plot(USdata.Quarters , [USdata.CPI  USdata.UR])
	
	restyle!(pp1, name = ["Unemployment (𝑈)", "Inflation (𝝅)"])
	
	relayout!(pp1, 
		height = 460, 
		legend=attr(x=0.03, y=0.95), legend_bgcolor="rgba(0,0,0,0)",
		title_text= "Unemployment and inflation in the US: 1960Q1-2023Q4", title_x = 0.5, hovermode="x", font_family="Calibri", font_size=14, 
		xaxis_range = ["1959-04-01" , "2024-10-01"])
	
	pp1
end


# ╔═╡ c7707bee-609b-4a8c-b154-747a279e8e21
#savefig(pp1, "az.pdf", width = 1000)

# ╔═╡ 93580fd5-721b-4183-9e10-2d512eb9e772
begin
	trace_01 = scatter(;x= USdata.Quarters, y= USdata.CPI, mode="lines",
				marker_symbol="circle", line_width= 2,
            	marker_color = "BlueViolet",  name = "Inflation (𝝅)", yaxis = "y2",
				fillcolor = "")#, text = period1_6)

	trace_02 = scatter(;x= USdata.Quarters, y= USdata.UR, mode="lines",
				marker_symbol="circle", line_width= 2,
            	marker_color = "DarkBlue", name = "Unemployment (𝑈)", yaxis = "y2",
				fillcolor = "")#, text = period1_6)

	trace_03 = scatter(;x= USdata.Quarters, y= USdata.GDP, mode="lines",
				marker_symbol="circle", marker_size="4",line_width= 2,
            	marker_color = "maroon",  name = "GDP (𝒚)") #, text = period1_6)

	layout_03 = Layout(; title_text= "GDP, Unemployment and Inflation in the US: 1960Q1-2023Q4",
	 					title_x=0.5, hovermode="x",	
						#titlefont_size = 16, 
						height=460, 
						font_family="Calibri", font_size=14,
						legend=attr( x=0.02, y=0.94), legend_bgcolor="rgba(0,0,0,0)",

			xaxis = attr(
                		title = "Quarterly obervations", 
						hovermode = "x",
						showgrid = true),
						xaxis_range = ["1959-04-01" , "2024-10-01"],

			yaxis1 = attr(
						title = " Real GDP",
						titlefont_color=  "maroon",
						tickfont_color = "maroon",
						#overlaying = "y",
						#side = "right",
						#yaxis1_tickvals = [1.5, 2.5, 3.5, 5.5],
						showgrid= false,
						zeroline=true,
						yaxis1_range=[1.5 , 5.5],
    					#dtick = 5.6 / 7
						  ),
			yaxis2 = attr(
    					title = "Unemployment , Inflation",
    					titlefont_color=  "blue",
    					tickfont_color = "blue",
    					overlaying = "y",
    					side = "right",
						showgrid = true,
						zeroline=false,
						yaxis2_range=[-25 , 5],
						#dtick = 30 / %
						  )
						)
	p_01 = plot([trace_01, trace_02, trace_03], layout_03)
	
end

# ╔═╡ 6f3bab31-f65f-4ca2-9e45-4cc3f630c608
md"""
Let us insert a new column, which will be named as **lnGDP**
"""

# ╔═╡ 0f04c187-bc39-40dc-8832-d5d0f8b62120
insertcols!(USdata, :lnGDP => log.(USdata.GDP))

# ╔═╡ f1a83428-3842-4eb4-94b7-82c4a5f58966
#insertcols!(USdata, :lnINV => log.(USdata.INV))

# ╔═╡ 809d9a65-60f8-4281-9d3a-52ba40e913b8
#select(USdata, Not(:lnINV))

# ╔═╡ 86c9852d-e35b-487e-b12a-3576591497ad
#select(USdata, Not(:FFR))

# ╔═╡ 37b8be8f-9327-462a-974a-2c4bf5328880
USdata2 = select(USdata, :CPI, :UR, :GDP, :lnGDP)

# ╔═╡ 9181790e-4783-48c6-be76-b2e789d9fc9a
md"""
### The HP filter
"""

# ╔═╡ b516b76d-9c31-4229-9757-60d0e65b33a0
md"""
The following cell creates a function that calculates the HP filter using the data set we called "USdata2". If one changes the name of the data set, the name of this variable has to be changed in accordance with the hp_filter function below.
"""

# ╔═╡ 9147a10d-4cd5-470a-b82e-1eb225370008
begin
	function hp_filter(USdata2::Array{Float64,1}, lambda::Float64)
    	n = length(USdata2)
    	@assert n >= 4
    	diag2 = lambda*ones(n-2)
    	diag1 = [ -2lambda; -4lambda*ones(n-3); -2lambda ]
    	diag0 = [ 1+lambda; 1+5lambda; (1+6lambda)*ones(n-4); 1+5lambda; 1+lambda ]
    	D = spdiagm(-2 => diag2, -1 => diag1, 0 => diag0, 1 => diag1, 2 => diag2)
    	D\USdata2
	end
end

# ╔═╡ 793a93dc-c9d7-48ec-bc64-a4a67e710158
md"""
To make calculating the HP filter very quickly understood from a computational point of view, we will first conduct the calculation for one single variable: logGDP. Notice that logGDP is the fourth column in our data set "USdata2". So we can use `USdata2[:,4]` in the code below, where 4 indicates that we will perform the computation on that column, or we can use the name of that column: `USdata2.lnGDP`. Then, we will generalize the exercise to all variables (that is, all columns in "USdata2"), which requires the use of `USdata2[:,n]` with $n$ as the columns: $n=1,2,3,4$.
"""

# ╔═╡ d3296808-90cc-4c14-86ac-12718d1d9973
md"""
### HP filter on a single variable
"""

# ╔═╡ bd2321e2-70f5-49ba-9b1f-fca9b3ddbcbd
md"""
Let us calculate the HP filter for the logarithmic value of GDP (column 4 in the data set `USdata2`). We can do it either by indicating the **column name** (USdata2.lnGDP) we want to filter, or by indicating of the **number** of the column we want to filter (USdata2[:,4]).
"""

# ╔═╡ 9f4c0a5a-92dc-46bd-9f6a-d980d210cb3f
md"""
### The HP filter for an entire data set
"""

# ╔═╡ c7a28f02-6e49-4894-9955-66d50de043e3
md"""
##### Obtaining the trend
"""

# ╔═╡ 48d1a9d6-ef89-463a-a768-c56420972d3e
md"""
We can check what the loop created for us by typing `hp_trend` and running this code. We get a matrix with 256 rows and 4 columns.
"""

# ╔═╡ 69c1d54d-9773-4b2e-894b-70de101d1926
md"""
According to the column names of our data set `USdata2`, the first column of the matrix above is associated with the variable CPI, the second with the variable UR, the third with GDP, and the last with the variable lnGDP. We can plot our Hodrick-Prescott filter using the data in the matrix above and the variable names associated with each column. 

There is a second possibility: we can create a new data frame from the data included in the matrix above and explicitly name the columns in this new data frame. This second approach is better than the first because it reduces the risk of mixing up the column names when plotting or when performing other tasks. Next, we show how to create a new data frame from the abovementioned matrix. 
"""

# ╔═╡ ce14ef7e-de0b-4032-9631-78e077485f6b
md"""
##### Obtaining the cycles
"""

# ╔═╡ 8f2eb52e-539c-431e-a2be-d99b4da7e9a0
md"""
#### Plotting
"""

# ╔═╡ b5c30c98-5757-4ad9-a4c6-129b0b69e88c
md"""
`λ = `$(@bind λ Slider(0.0:400.0:40000.0, default=1600.0, show_value=true))
"""

# ╔═╡ d747423d-e45e-4b03-b604-f426b0ca64e4
begin	
	GDP_trend = hp_filter(USdata2.lnGDP, λ) 	# or #GDP_trend = hp_filter(USdata2[:,4], λ)	
	GDP_cycles = USdata2.lnGDP - GDP_trend		# or GDP_cycles = USdata2[:,4] - GDP_trend	
end;

# ╔═╡ da6cd1a0-29da-431a-aeb1-f19d161e2538
begin
	p10=plot(USdata.Quarters, [USdata2.lnGDP  GDP_trend], Layout(height = 450, hovermode = "x"))
	relayout!(p10, Layout(#font_size = 13, 
					height = 450,
					hovermode="x",
					xaxis_range = ["1959-04-01" , "2024-10-01"],
					xaxis_title = "Quarters",
	                yaxis_title = "Billion dollars (logarithmic)",
				   	title_text= "GDP versus GDP trend (US: 1960Q1-2023Q4)", title_x =0.5,
				   	titlefont_size="17"))
	restyle!(p10, name=["GDP", "HP_trend"])
	p10
end

# ╔═╡ 15b7e047-7d56-4dcb-a68f-1794f582dfbd
begin
	p11=plot(USdata.Quarters, GDP_cycles, kind="bar", marker_color = "Navy")
	relayout!(p11, Layout(#font_size = 13, 
						height = 450,
						hovermode="x",
						xaxis_range = ["1959-04-01" , "2024-10-01"],
						xaxis_title = "Quarters",
	                	yaxis_title = "Percentage deviations",
					   	title_text= "The Output-gap: H-P filtered business cycles (US: 1960Q1-2023Q4)}", title_x =0.5,
					   	titlefont_size="17"))
	restyle!(p11, name=["GDP", "HP_GDP"])
	p11
end

# ╔═╡ 3ceba09c-de39-4411-8efc-d16a765f22fc
begin
	hp_trend = zeros(256,4) 							# Pre-allocating space for the data
	
	for n = 1:4 										# begins the "for" loop
    	hp_trend[:,n] = hp_filter(USdata2[:,n], λ) 		# applies the loop to columns 1 to 4 of the data frame "USdata2"
	end													# ends the "for" loop
end

# ╔═╡ 8c4a9a0b-e34f-44e7-80d6-e3b176234f2b
hp_trend

# ╔═╡ 876f155d-bd0b-4c48-aa81-8c912e21ae0b
trends = DataFrame(hp_trend, [:CPI, :UR, :GDP, :lnGDP]) # creates a new data frame out of the hp_trend matrix

# ╔═╡ fc26b000-601d-426f-a775-8d3663155238
begin
	hp_cycles = zeros(256,4) 									# Pre-allocation 
	
	for n = 1:4 												# begins the "for" loop
    	hp_cycles[:,n] = USdata2[:,n] - hp_trend[:,n] 			# applies the loop
	end 														# ends the loop
end

# ╔═╡ d50be6bb-4948-4717-8f90-c47be873b85e
cycles = DataFrame(hp_cycles, [:CPI, :UR, :GDP, :lnGDP]) # creates a new data frame out of the hp_cycles matrix

# ╔═╡ a2f26c7f-6cad-46e5-b4d4-f2b20ce71e4f
plot(cycles.CPI)

# ╔═╡ 28f2c903-a27b-4ac8-8cb4-c9091776019b
begin
	trace_04 = scatter(; x= USdata.Quarters, y= cycles.CPI, mode = "markers+lines", 
				marker_symbol = "circle", marker_size = "5.5", line_width = 0.3,
            	marker_color = "darkblue", name = "CPI")

	layout_04= Layout(;title_text= "The inflation-gap in the US: 1960Q1-2023Q4",
				height= 450,
				title_x = 0.5, 
				titlefont_size = 16,
				hovermode = "x",	
            	xaxis = attr(
                	title = "Quarterly obervations",
                	tickformat = "%Y",
                	hoverformat = "%Y-Q%q",
                	tick0 = "1950/04/01",
                	dtick = "M120"),
        		
        		yaxis_title = "Percentage deviations",
				xaxis_range = ["1959-04-01" , "2024-10-01"],
        		yaxis_range=[-6, 7])
	
	p_02 = plot([trace_04], layout_04)
	#savefig(p_02, "p02.svg")
end

# ╔═╡ 7b6c40ab-c813-4e26-9f81-d2664fcd379c
begin
	trace_05 = scatter(;x= USdata.Quarters, y= cycles.UR, mode="markers+lines",
				marker_symbol="circle", marker_size="5.5",line_width= 0.3,
            	marker_color = "darkblue", name="UR", fillcolor = "darkblue")

	layout_05= Layout(;title_text= "Cyclical unemployment in the US: 1960Q1-2023Q4",
				font_family="Calibri", font_size=15, #titlefont_size = 18,
				height= 450,
				title_x = 0.5, hovermode = "x",
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1950/04/01",
                dtick = "M120"),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "% deviations",
				xaxis_range = ["1959-04-01" , "2024-10-01"],
        		yaxis_range=[-2, 9])
	p_03 = plot([trace_05], layout_05)
	#savefig(p_03, "p03.svg")
end

# ╔═╡ 0e52a463-ef5c-41ca-a8dc-900b27370f5f
plot(hp_cycles[:, 1])

# ╔═╡ 6b1b2d07-c4e9-480e-809d-fa34a8325541
begin
	trace_06 = scatter(;x= USdata.Quarters, y= cycles.lnGDP, mode="markers+lines",
				marker_symbol="circle", marker_size="5.5",line_width= 0.3,
            	marker_color = "Navy", name="CPI", fillcolor = "red")

	layout_06= Layout(;title_text= "Correct measure of the Output-gap in the US: 1960Q1-2023Q4",
				height= 450,
				title_x = 0.5, hovermode = "x",	titlefont_size = 16,
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1950/04/01",
                dtick = "M120"),
        		xaxis_range = ["1959-04-01" , "2024-10-01"],
        		#yaxis_title = "% deviations",
        		#yaxis_range=[-10, 4])
	)
	p_06 = plot([trace_06], layout_06)
	#savefig(p_06, "p06.svg")
end

# ╔═╡ 760790e0-f3d5-40c4-b2ee-ca1b8504051d
md"""
The following plot displays a common mistake when we apply the HP filter to a time series with a clear trend **_but do not apply logarithms_** to the original data. The cycles this procedure produces increase in volatility over time, which we do not intend to obtain with this exercise.
"""

# ╔═╡ db03fe06-4a63-4a31-9cf3-73969283bcb7
begin
	trace_07 = scatter(;x= USdata.Quarters, y = cycles.GDP, mode="lines",
				marker_symbol="circle", marker_size="5.5",line_width= 2,
            	marker_color = "purple", name="CPI", fillcolor = "red")

	layout_07= Layout(;title_text= "Incorrect measure of the Output-gap in the US: 1960Q1-2023Q4",
				height = 450,
				title_x = 0.5, hovermode = "x",	titlefont_size = 16,
				xaxis_range = ["1959-04-01" , "2024-10-01"],
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1950/04/01",
                dtick = "M120"),
        		#xaxis_range = [100, 200],
        		yaxis_title = " Billion dollars")
        		#yaxis_range=[-10, 4])
	p_07 = plot([trace_07], layout_07)
	#savefig(p_07, "p07.svg")
end

# ╔═╡ 60d70eca-eaaf-436d-8096-89e0e5fa778e
md"""
No logarithms applied in the following plot. Nevertheless, the output is correct because the original data does not have a clear trend.
"""

# ╔═╡ bc52abd7-d184-4adb-8aa5-c51d8099b8c9
begin
	trace_08 = scatter(;x= USdata.Quarters, y = trends.UR, mode="lines+markers",
				marker_symbol="circle", marker_size="4.5",line_width= 0.3,
            	marker_color = "darkred", name="NRU (𝑼ₙ)", fillcolor = "gray")

	trace_09 = scatter(;x= USdata.Quarters, y = USdata.UR, mode="lines+markers",
				marker_symbol="circle", marker_size="5",line_width= 0.3,
            	marker_color = "blue", name="Unemployment (𝑼)", fillcolor = "blue")

	layout_09= Layout(; height = 450, 
				font_family="Calibri", font_size=15, titlefont_size = 18,
	title_text= "Unemployment vs the Natural Rate of Unemployment (trend) in the US: 1960Q1-2023Q4",
				legend=attr( x=0.02, y=0.94), #legend_bgcolor="rgba(0,0,0,0)",
				title_x = 0.5, hovermode = "x",	
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1950/04/01",
                dtick = "M120"),
				xaxis_range = ["1959-04-01" , "2024-10-01"],
        		yaxis_title = " Percentage points")
        		#yaxis_range=[-10, 4])
	p_09 = plot([trace_08, trace_09], layout_09)
	#savefig(p_09, "p09.svg")
end

# ╔═╡ 0f3fc6f9-f2d3-48a2-a0fb-4f8bcfddb84e
#savefig(p_09, "p09.svg", width = 900)

# ╔═╡ 94cfd9cf-586d-4a7a-b5ee-a0cb4c3ee288
md"""
## 2. Impulse Response Functions
"""

# ╔═╡ 8c4238a8-2ca1-4a88-aee1-95c2afafb871
md"""
### AR(1)
"""

# ╔═╡ 88e1d017-2272-41a0-b97c-10ffc1c7f55b
md"""
Consider the following simple auto-regressive process of order 1 (AR(1)):

$$y_{t+1}={\color{blueviolet}a} \cdot y_{t} + \varepsilon_{t+1} \tag{1}$$

where is $a$ a parameter. 

- Assume that for $t \in(1,n)$: 
 	
$$y_{1}=0 \ ; \ \varepsilon_{2}=1 \ ; \ \varepsilon_{t}=0 \ , \ \forall t\neq2$$

- This implies that at $t=2$, $y_2 = 1$, but what happens next, if there are no more shocks?
- The IRF of $y$ provides the answer.
- The dynamics of $y$ will depend crucially on the value of $\color{blueviolet}a$. Six examples:
  $$\color{blueviolet} a = \{0,0.5,0.9, 0.99, 1, 1.01\}$$

To make it easier, we start with the case of $a=0.5$. Then, we move over to all the remaining cases.
"""

# ╔═╡ 17e30d59-0856-42e1-bf47-20e6d0e37057
begin 												# this block uses normal characters 
	n = 20 											# number of iterations
	y1 = 0.0 										# setting the initial condition
	l = length(y1)									# length of of the initial condition
	ε = zeros(1,n); ε[1,2] = 1 						# definig the shock
	a = 0.5 										# setting the value of a
	y = [y1  zeros(l, n-1)] 						# pre-allocation for y

		for t = 1:n-1 								# begins the "for" loop
	   		y[t+1] =  a * y[t] + ε[t+1]             # y(t+1) = a + y(t) + ε(t+1)
		end 										# ends the loop
	
	plot(y', mode="markers+lines",
			marker_symbol="circle", marker_size="6", line_width= 0.3, marker_color = "Blue", 
			Layout(title_text = "A stochastic AR(1) process", title_x = 0.5, hovermode = "x", height = 450)
		)
end

# ╔═╡ 9ad7d089-0885-4087-9ee0-4f9c35325441
begin 														# this block uses special characters of class: \bb
	ℕ = 30 													# number of iterations
	𝕒 = (0.0, 0.5 , 0.9 , 0.99 , 1.0 , 1.01) 				# four different values for parameter α
	𝕪1 = [0.0 ; 0.0 ; 0.0 ; 0.0 ; 0.0 ; 0.0 ] 				# setting the initial conditions 
	𝕃 = length(𝕪1) 											# number of initial conditions
	𝕖 = zeros(𝕃,31) 										# pre-allocation for ε
	𝕖[:,2] = [1.0 ; 1.0 ; 1.0 ; 1.0 ; 1.0 ; 1.0] 			# setting the shock
	𝕪 = [𝕪1  zeros(𝕃, ℕ-1)] 					    		# pre-allocation for 𝓎
	  	    
		for 𝕜 in eachindex(𝕒) 									# nested "for" loop
			for 𝕥 = 1 : ℕ-1 
				𝕪[𝕜,𝕥+1] = 𝕒[𝕜] .* 𝕪[𝕜,𝕥] .+ 𝕖[𝕜,𝕥+1]      	# y(t+1) = a + y(t) + ε(t+1), for k=6 values of a
			end
		end 												# ends the nested loop
	
	p101 = plot(𝕪',mode = "markers+lines", line_width="0.5", marker_size ="6")
	restyle!(p101, name=[ "𝒂 = 0" , "𝒂 = 0.5" , "𝒂 = 0.9" , "𝒂 = 0.99", "𝒂 = 1.0" , "𝒂 = 1.01" ])
	restyle!(p101, line_color = [ "blue" , "red" , "grey" , "purple" , "darkblue", ""])
	relayout!(p101, title_text = "Impulse Response Functions (IRF) from an AR(1) process", title_x = 0.5, hovermode ="x", height = 500)
	p101
end

# ╔═╡ 32ed885e-cf2f-4cee-b8df-cc81a463d429
md"""
### AR(2)
"""

# ╔═╡ 144c6ac4-4159-4520-bddd-3b75d624a426
md"""
- Consider a more sophisticated case, an AR(2):

$$y_{t+1}= {\color{red}a} y_{t}+{\color{blueviolet}b} y_{t-1} +  \varepsilon_{t+1} \ , \quad \varepsilon_{t} \sim N(0,1) \tag{2}$$
- Assume that for $t \in(1,n)$: 

$$y_{1}=0 \ ; \ y_2=0 \ ; \ \varepsilon_{3}=1 \ ; \ \varepsilon_{t}=0 \ , \ \forall t\neq 3$$

- This implies that at
  
  $$t=3 \ , \ \varepsilon_{3}=1 \ \Rightarrow \ y_3 = 1.$$
  
  What happens next, if there are no more shocks? The IRF of $y$ provides the answer.
- The dynamics of $y$ will depend on the values of $\color{blueviolet}a$ and $\color{red}b$. For simplicity consider:
 
$${\color{red}b = -0.9} \ ; \color{blueviolet} a = \{1.85,1.895,1.9,1.9005\}$$

"""

# ╔═╡ 7bb3d405-e175-4865-a0a0-856a69aeb145
begin 															# this block uses special characters of class: \bf
	𝐍 = 99  													# number of iterations
	𝐚 = (1.85 , 1.895 , 1.9 , 1.9005) 							# four different values for parameter a 
	𝐲1 = [ 0.0  0.0 ; 0.0  0.0 ; 0.0  0.0 ; 0.0  0.0 ] 	   		# initial conditions
	𝐋 = length(𝐲1) 												# number of initial conditions
	𝛜 = zeros(𝐋-4 , 𝐍+1) 										# pre-allocation for ε
	𝛜[:,3] = [1.0 ; 1.0 ; 1.0 ; 1.0 ] 							# setting the shock
	𝐲 = [𝐲1  zeros(𝐋-4, 𝐍-1)] 					    			# pre-allocation for 𝓎
	  	    
	𝐛 = -0.9 													# setting parameter b

		for k in eachindex(𝐚) 									# nested for-loop
			for t = 1 : 𝐍-1 
				𝐲[k,t+2]  = 𝐚[k] .* 𝐲[k,t+1] + 𝐛 .* 𝐲[k,t] .+ 𝛜[k,t+2] 
			end
		end
	p110 = plot(𝐲',mode = "markers+lines", line_width="0.5", marker_size ="5")
	restyle!(p110, name=["𝒂 = 1.85" , "𝒂 = 1.895" , "𝒂 = 1.9" , "𝒂 = 1.9005"])
	restyle!(p110, line_color =["blue" , "navy" , "gray" , "orange"])
	relayout!(p110, title_text = "Impulse Response Functions (IRF) from an AR(2) process", title_x = 0.5, hovermode = "x", height = 500,
	#legend=attr( x=0.995, y=0.95)
	)
	p110
end

# ╔═╡ 23786f58-0f82-4a23-914c-0bb40c63f4e7
md"""
### VAR(3)
"""

# ╔═╡ 5107f7cd-d9da-472a-bcba-9f9dd04107f7
md"""
A similar reasoning can be applied to our rather more general model:

$$X_{t+1}= A + BX_{t}+ C\varepsilon_{t+1} \tag{3}$$

where $B , C$ are $n\times n$ matrices, while $X_{t+1}, X_{t}, A, \varepsilon_{t+1}$ are $n\times 1$ vectors. 

Consider the following VAR(3) model:

$$X_{t+1}=\left[\begin{array}{c}
z_{t+1} \\
w_{t+1} \\
v_{t+1}
\end{array}\right] \tag{4}$$

---

In this example we take matrices $A, B$ and $C$ given by: 

$$\begin{gathered}
A=\left[\begin{array}{c}
0.0  \\
0.0  \\
0.0 
\end{array}\right], \quad

B=\left[\begin{array}{rrr}
0.97 & 0.10 & -0.05 \\
-0.3 & 0.8 & 0.05 \\
0.01 & -0.04 & 0.96
\end{array}\right] , \quad


C=\left[\begin{array}{lll}
{\color{blueviolet}1.0} & 0.0 & 0.0 \\
0.0 & 0.0 & 0.0 \\
0.0 & 0.0 & 0.0
\end{array}\right] .
\end{gathered} \tag{5}$$

- The initial state of our system (or its initial conditions) are: $z_{1}=0, w_{1}=0$ and $v_{1}=0$, that is:

$$X_{1}=[0,0,0] \tag{6}$$

- The shock only hits the variable $z_t$ $($notice the blue entry in matrix $C)$, and we assume that the shock occurs in period $t=3$.
- What happens to the dynamics of the three endogenous variables? See next figure.
"""

# ╔═╡ c1343361-86dd-447a-8ca1-9a2a506bf78d
md"""
We start by inserting the entries of matrices $A,B,C$, according to the data above, into the notebook:
"""

# ╔═╡ 62c41a14-4cdd-488c-a653-0159f10fef1d
begin
	𝐴 = zeros(3,1)											        # matrix A 
	𝐵 = [ 0.97  0.10 -0.05 ; -0.3  0.8  0.05 ; 0.01 -0.04  0.96 ] 	# matrix B
	𝐶 = [ 1.0  0.0  0.0 ; 0.0  0.0  0.0 ; 0.0  0.0  0.0 ] 			# matrix C
end;

# ╔═╡ 4ae19cc8-4a3b-4118-af0a-d179f385916b
md"""
`𝑧1 = `$(@bind 𝑧1 Slider(0.0:1.0:5.0, default= 1.0, show_value=true))
"""

# ╔═╡ 7d2c6085-fd4b-42c7-a27c-b39f6ddf8290
begin 																# this block uses special characters of class: \it
	𝑁 = 40                         									# number of iterations
	𝑋1 = [0.0 ; 0.0  ; 0.0 ] 										# initial conditions
	𝐿 = length(𝑋1)													# the number of initial conditions 
	𝜀 = zeros(𝐿, 𝑁) 												# pre-allocation for ε
	𝜀[1,3] = 𝑧1 													# setting the shock (using the slider above)	
	𝑋 = [𝑋1  zeros(𝐿, 𝑁-1)]; 										# preallocation fro
		
		for t = 1 : 𝑁-1 
			𝑋[:, t+1] = 𝐴 + 𝐵*𝑋[:, t] + 𝐶 * 𝜀[:, t+1] 
		end
		
	
	p12 = plot(𝑋', mode = "markers+lines",
			marker_symbol = "circle", marker_size = "6", line_width = 0.3, 
			Layout(title_text = "A VAR(3) process", title_x = 0.5, hovermode = "x"))
	restyle!(p12, 1:3, name = ["𝒛(𝒕)" , "𝒘(𝒕)" , "𝒗(𝒕)"])
	restyle!(p12, 1:3, line_color = ["blue" , "gray" , "navy"])
	relayout!(p12, title_text = "Impulse Response Functions (IRF) from a VAR(3) process", title_x = 0.5, 
		height = 500, legend=attr( x=0.9, y=0.95),)
	p12
	#savefig(p12, "VAR_3.svg", width = 1000)
end

# ╔═╡ dfd8225d-f7de-4740-86de-69a70227afee
md"""

### One shock, versus a sequence of shocks

"""

# ╔═╡ 12bad102-fae5-4d2d-8676-630c59f61956
md"""
`𝒂 = `$(@bind 𝒂 Slider(0.0:0.1:1.0, default= 0.6, show_value=true))
"""

# ╔═╡ 4101ea53-eefa-4af8-b488-3ed431a1cee8
begin 												# this block uses special characters of class: \bi
	𝒏 = 200  										# large 𝒏 to fully observe the stochastic dynamics
	𝒚1 = [0.0] 										# initial conditions
	𝑳 = length(𝒚1) 									# number of initial conditions 
	𝜺 = randn(1,𝒏) 									# definig the shock
	#𝒂 												# parameter 𝒂 is defined by the 𝒂-slider above
	𝒚 = [𝒚1  zeros(𝑳, 𝒏-1)] 						# pre-allocation for y

	 	for t = 1 : 𝒏-1 							# begins the "for" loop
	     	𝒚[t+1] = 𝒂 * 𝒚[t] + 𝜺[t+1] 				# y(t+1) = a + y(t) + ε(t+1)
	 	end 										# ends the loop
	
	p13 = plot(𝒚', mode="markers+lines",
			marker_symbol="circle", marker_size="6",line_width= 0.3, marker_color = "Blue", 
			Layout(title_text = "A stochastic AR(1) process with mean=0 and a sequence of shocks", title_x = 0.5, 
				hovermode = "x", height = 500))
	#savefig(p13, "AR200_1.svg", width = 1000)
end

# ╔═╡ fc0fa874-59f8-49c7-b734-4bc38bb60908
begin
	auto_yₛ = autocor(𝒚')
	plot(auto_yₛ, mode = "markers+lines", marker_size = "6", line_color = "BlueViolet", marker_color = "Blue", line_width = 0.6,
	Layout(title_text = "Autocorrelation function: stochastic AR(1) process", title_x=0.5, hovermode = "x",
	xaxis_title = "lag", yaxis_title = "Autocorrelation", height = 450)) 
end

# ╔═╡ 319d1622-6458-42a4-8e87-4aae991cab61
md"""
## 3. How to produce a nice plot
"""

# ╔═╡ 9aa0ccd4-d986-4e6a-ba7b-5652e6edcc6f
begin
	# to produce a plot named pic, with Quarters in the x-axis, and CPI and FFR in the y-axis, using the USdata dataframe we should start by doing:
	
	pic = plot(USdata.Quarters, [USdata.CPI USdata.FFR]) 

	# Then we can add new features to our plot by using two functions in Plotly: relayout! and restyle!
	
	relayout!(pic, 										# changes the layout of our plot pic
		title_text = "My fancy title" , 				# adds a title title
		title_x= 0.5 ,  								# centers the title
		height = 500 , 									# allows different height values
		width = 980 , 									# allows different width values
		#hovermode = "x",								# sets the hover to the x-axis with a box
		hovermode = "x unified",  						# sets the hover to the x-axis with a bar 
		xaxis_title = "Quarterly observations" ,        # adds a title to the x-axis 
		yaxis_title = "Percentage points" ,           	# adds a title to the x-axis
		xaxis_range = ["1959-04-01" , "2024-10-01"],  	# allows us to change the x-axis range
		yaxis_range = [ -2 , 18 ],    					# allows us to change the y-axis range
		font_family="Calibri", 							# sets customizable font families
		font_size= 15   								# sets customizable font size
	)
	
	restyle!(pic, 										# adds/changes the attributes on our  plot pic
		name = ["CPI"  "FFR"] , 						# name our variables as we wish
		mode = "markers+lines" , 						# allows us to set markers, lines, or markers+lines
		marker_color = ["Blue"  "Red"] ,				# change the line colors as we wish
		marker_size =  5 , 								# allows us to use different marker sizes
		line_width = 0.5 , 								# allows us to use different line widths
	)
	
	pic 												# Produces the plot with the new layout and the new attributes
	
end

# ╔═╡ b09f96de-e2dc-4306-8547-226ec47372a5
md"""
## 4. How to create CSV files
"""

# ╔═╡ 85191a8a-d381-4580-8c99-0423452f469d
md"""
NEXT.
"""

# ╔═╡ 2fd32ff5-13f3-41cc-806e-d7e375a4c4ce
md"""
## Auxiliary cells (do not delete)
"""

# ╔═╡ 243c8313-dfa2-4f96-ad95-2e9c815a15ea
TableOfContents()

# ╔═╡ 8be3d1b0-cf85-48a6-966f-3203741f13d3
#html"""<style> #main{ max-width: 1000px; align-self: flex-start;  margin-left: 600px;}"""

# ╔═╡ fd2c9069-9b3e-4b5f-8e1a-75728c43a387
html"""
<style>
	@media screen {
		main {
			margin: 0 auto;
			max-width: 1600px;
    		padding-left: max(200px, 10%);
    		padding-right: max(383px, 10%); 
            # 383px to accomodate TableOfContents(aside=true)
		}
	}
</style>
"""

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
LaTeXStrings = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
LinearAlgebra = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
NLsolve = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
StatsBase = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
TimeSeries = "9e3dc215-6440-5c97-bce1-76c03772f85e"

[compat]
CSV = "~0.10.9"
DataFrames = "~1.7.0"
LaTeXStrings = "~1.4.0"
NLsolve = "~4.5.1"
PlotlyKaleido = "~2.2.2"
PlutoPlotly = "~0.4.4"
PlutoUI = "~0.7.50"
StatsBase = "~0.34.2"
TimeSeries = "~0.23.1"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.11.3"
manifest_format = "2.0"
project_hash = "ac6d687623553018ed81829cdcc04d17b23e4ad3"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "6e1d2a35f2f90a4bc7c2ed98079b2ba09c35b83a"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.3.2"

[[deps.Adapt]]
deps = ["LinearAlgebra", "Requires"]
git-tree-sha1 = "cd8b948862abee8f3d3e9b73a102a9ca924debb0"
uuid = "79e6a3ab-5dfb-504d-930d-738a2a938a0e"
version = "4.2.0"

    [deps.Adapt.extensions]
    AdaptSparseArraysExt = "SparseArrays"
    AdaptStaticArraysExt = "StaticArrays"

    [deps.Adapt.weakdeps]
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.AliasTables]]
deps = ["PtrArrays", "Random"]
git-tree-sha1 = "9876e1e164b144ca45e9e3198d0b689cadfed9ff"
uuid = "66dad0bd-aa9a-41b7-9441-69ab47430ed8"
version = "1.1.3"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.2"

[[deps.ArrayInterface]]
deps = ["Adapt", "LinearAlgebra"]
git-tree-sha1 = "017fcb757f8e921fb44ee063a7aafe5f89b86dd1"
uuid = "4fba245c-0d91-5ea0-9b3e-6abc04ee57a9"
version = "7.18.0"

    [deps.ArrayInterface.extensions]
    ArrayInterfaceBandedMatricesExt = "BandedMatrices"
    ArrayInterfaceBlockBandedMatricesExt = "BlockBandedMatrices"
    ArrayInterfaceCUDAExt = "CUDA"
    ArrayInterfaceCUDSSExt = "CUDSS"
    ArrayInterfaceChainRulesCoreExt = "ChainRulesCore"
    ArrayInterfaceChainRulesExt = "ChainRules"
    ArrayInterfaceGPUArraysCoreExt = "GPUArraysCore"
    ArrayInterfaceReverseDiffExt = "ReverseDiff"
    ArrayInterfaceSparseArraysExt = "SparseArrays"
    ArrayInterfaceStaticArraysCoreExt = "StaticArraysCore"
    ArrayInterfaceTrackerExt = "Tracker"

    [deps.ArrayInterface.weakdeps]
    BandedMatrices = "aae01518-5342-5314-be14-df237901396f"
    BlockBandedMatrices = "ffab5731-97b5-5995-9138-79e8c1846df0"
    CUDA = "052768ef-5323-5732-b1bb-66c8b64840ba"
    CUDSS = "45b445bb-4962-46a0-9369-b4df9d0f772e"
    ChainRules = "082447d4-558c-5d27-93f4-14fc19e9eca2"
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    GPUArraysCore = "46192b85-c4d5-4398-a991-12ede77f4527"
    ReverseDiff = "37e2e3b7-166d-5795-8a7a-e32c996b4267"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    StaticArraysCore = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
    Tracker = "9f7883ad-71c0-57eb-9f7f-b5c9e6d3789c"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"
version = "1.11.0"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"
version = "1.11.0"

[[deps.BaseDirs]]
git-tree-sha1 = "cb25e4b105cc927052c2314f8291854ea59bf70a"
uuid = "18cc8868-cbac-4acf-b575-c8ff214dc66f"
version = "1.2.4"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "PrecompileTools", "SentinelArrays", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "deddd8725e5e1cc49ee205a1964256043720a6c3"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.15"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "962834c22b66e32aa10f7611c08c8ca4e20749a9"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.8"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "PrecompileTools", "Random"]
git-tree-sha1 = "403f2d8e209681fcbd9468a8514efff3ea08452e"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.29.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "b10d0b65641d57b8b4d5e234446582de5047050d"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.5"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "Requires", "Statistics", "TensorCore"]
git-tree-sha1 = "a1f44953f2382ebb937d60dafbe2deea4bd23249"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.10.0"
weakdeps = ["SpecialFunctions"]

    [deps.ColorVectorSpace.extensions]
    SpecialFunctionsExt = "SpecialFunctions"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "362a287c3aa50601b0bc359053d5c2468f0e7ce0"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.11"

[[deps.CommonSubexpressions]]
deps = ["MacroTools"]
git-tree-sha1 = "cda2cfaebb4be89c9084adaca7dd7333369715c5"
uuid = "bbf7d656-a473-5ed7-a52c-81e309532950"
version = "0.3.1"

[[deps.Compat]]
deps = ["TOML", "UUIDs"]
git-tree-sha1 = "8ae8d32e09f0dcf42a36b90d4e17f5dd2e4c4215"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.16.0"
weakdeps = ["Dates", "LinearAlgebra"]

    [deps.Compat.extensions]
    CompatLinearAlgebraExt = "LinearAlgebra"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.1.1+0"

[[deps.ConstructionBase]]
git-tree-sha1 = "76219f1ed5771adbb096743bff43fb5fdd4c1157"
uuid = "187b0558-2788-49d3-abe0-74a17ed4e7c9"
version = "1.5.8"

    [deps.ConstructionBase.extensions]
    ConstructionBaseIntervalSetsExt = "IntervalSets"
    ConstructionBaseLinearAlgebraExt = "LinearAlgebra"
    ConstructionBaseStaticArraysExt = "StaticArrays"

    [deps.ConstructionBase.weakdeps]
    IntervalSets = "8197267c-284f-5f27-9208-e0e47529a953"
    LinearAlgebra = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "abe83f3a2f1b857aac70ef8b269080af17764bbe"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.16.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "DataStructures", "Future", "InlineStrings", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrecompileTools", "PrettyTables", "Printf", "Random", "Reexport", "SentinelArrays", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "fb61b4812c49343d7ef0b533ba982c46021938a6"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.7.0"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "1d0a14036acb104d9e89698bd408f63ab58cdc82"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.20"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"
version = "1.11.0"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.DiffResults]]
deps = ["StaticArraysCore"]
git-tree-sha1 = "782dd5f4561f5d267313f23853baaaa4c52ea621"
uuid = "163ba53b-c6d8-5494-b064-1a9d43ac40c5"
version = "1.1.0"

[[deps.DiffRules]]
deps = ["IrrationalConstants", "LogExpFunctions", "NaNMath", "Random", "SpecialFunctions"]
git-tree-sha1 = "23163d55f885173722d1e4cf0f6110cdbaf7e272"
uuid = "b552c78f-8df3-52c6-915a-8e097449b14b"
version = "1.15.1"

[[deps.Distances]]
deps = ["LinearAlgebra", "Statistics", "StatsAPI"]
git-tree-sha1 = "c7e3a542b999843086e2f29dac96a618c105be1d"
uuid = "b4f34e82-e78d-54a5-968a-f98e89d6e8f7"
version = "0.10.12"

    [deps.Distances.extensions]
    DistancesChainRulesCoreExt = "ChainRulesCore"
    DistancesSparseArraysExt = "SparseArrays"

    [deps.Distances.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[deps.Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"
version = "1.11.0"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates"]
git-tree-sha1 = "2ec417fc319faa2d768621085cc1feebbdee686b"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.23"
weakdeps = ["Mmap", "Test"]

    [deps.FilePathsBase.extensions]
    FilePathsBaseMmapExt = "Mmap"
    FilePathsBaseTestExt = "Test"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"
version = "1.11.0"

[[deps.FiniteDiff]]
deps = ["ArrayInterface", "LinearAlgebra", "Setfield"]
git-tree-sha1 = "f089ab1f834470c525562030c8cfde4025d5e915"
uuid = "6a86dc24-6348-571c-b903-95158fe2bd41"
version = "2.27.0"

    [deps.FiniteDiff.extensions]
    FiniteDiffBandedMatricesExt = "BandedMatrices"
    FiniteDiffBlockBandedMatricesExt = "BlockBandedMatrices"
    FiniteDiffSparseArraysExt = "SparseArrays"
    FiniteDiffStaticArraysExt = "StaticArrays"

    [deps.FiniteDiff.weakdeps]
    BandedMatrices = "aae01518-5342-5314-be14-df237901396f"
    BlockBandedMatrices = "ffab5731-97b5-5995-9138-79e8c1846df0"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "05882d6995ae5c12bb5f36dd2ed3f61c98cbb172"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.5"

[[deps.ForwardDiff]]
deps = ["CommonSubexpressions", "DiffResults", "DiffRules", "LinearAlgebra", "LogExpFunctions", "NaNMath", "Preferences", "Printf", "Random", "SpecialFunctions"]
git-tree-sha1 = "a2df1b776752e3f344e5116c06d75a10436ab853"
uuid = "f6369f11-7733-5829-9624-2563aa707210"
version = "0.10.38"

    [deps.ForwardDiff.extensions]
    ForwardDiffStaticArraysExt = "StaticArrays"

    [deps.ForwardDiff.weakdeps]
    StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"
version = "1.11.0"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "179267cfa5e712760cd43dcae385d7ea90cc25a4"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.5"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "7134810b1afce04bbc1045ca1985fbe81ce17653"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.5"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "b6d6bfdd7ce25b0f9b2f6b3dd56b2673a66c8770"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.5"

[[deps.InlineStrings]]
git-tree-sha1 = "6a9fde685a7ac1eb3495f8e812c5a7c3711c2d5e"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.3"

    [deps.InlineStrings.extensions]
    ArrowTypesExt = "ArrowTypes"
    ParsersExt = "Parsers"

    [deps.InlineStrings.weakdeps]
    ArrowTypes = "31f734f8-188a-4ce0-8406-c8a06bd891cd"
    Parsers = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"
version = "1.11.0"

[[deps.InvertedIndices]]
git-tree-sha1 = "6da3c4316095de0f5ee2ebd875df8721e7e0bdbe"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.3.1"

[[deps.IrrationalConstants]]
git-tree-sha1 = "e2222959fbc6c19554dc15174c81bf7bf3aa691c"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.2.4"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JLLWrappers]]
deps = ["Artifacts", "Preferences"]
git-tree-sha1 = "a007feb38b422fbdab534406aeca1b86823cb4d6"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.7.0"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "31e996f0a15c7b280ba9f76636b3ff9e2ae58c9a"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.4"

[[deps.Kaleido_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "43032da5832754f58d14a91ffbe86d5f176acda9"
uuid = "f7e6163d-2fa5-5f23-b69c-1db539e41963"
version = "0.2.1+0"

[[deps.LaTeXStrings]]
git-tree-sha1 = "dda21b8cbd6a6c40d9d02a73230f9d70fed6918c"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.4.0"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.4"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "8.6.0+0"

[[deps.LibGit2]]
deps = ["Base64", "LibGit2_jll", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"
version = "1.11.0"

[[deps.LibGit2_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll"]
uuid = "e37daf67-58a4-590a-8e99-b0245dd2ffc5"
version = "1.7.2+0"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.11.0+1"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"
version = "1.11.0"

[[deps.LineSearches]]
deps = ["LinearAlgebra", "NLSolversBase", "NaNMath", "Parameters", "Printf"]
git-tree-sha1 = "e4c3be53733db1051cc15ecf573b1042b3a712a1"
uuid = "d3d80556-e9d4-5f37-9878-2ab0fcc64255"
version = "7.3.0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
version = "1.11.0"

[[deps.LogExpFunctions]]
deps = ["DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "13ca9e2586b89836fd20cccf56e57e2b9ae7f38f"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.29"

    [deps.LogExpFunctions.extensions]
    LogExpFunctionsChainRulesCoreExt = "ChainRulesCore"
    LogExpFunctionsChangesOfVariablesExt = "ChangesOfVariables"
    LogExpFunctionsInverseFunctionsExt = "InverseFunctions"

    [deps.LogExpFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    ChangesOfVariables = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
    InverseFunctions = "3587e190-3f89-42d0-90ee-14403ec27112"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"
version = "1.11.0"

[[deps.MIMEs]]
git-tree-sha1 = "1833212fd6f580c20d4291da9c1b4e8a655b128e"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "1.0.0"

[[deps.MacroTools]]
git-tree-sha1 = "72aebe0b5051e5143a079a4685a46da330a40472"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.15"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"
version = "1.11.0"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.6+0"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "ec4f7fbeab05d7747bdf98eb74d130a2a2ed298d"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.2.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"
version = "1.11.0"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2023.12.12"

[[deps.NLSolversBase]]
deps = ["DiffResults", "Distributed", "FiniteDiff", "ForwardDiff"]
git-tree-sha1 = "a0b464d183da839699f4c79e7606d9d186ec172c"
uuid = "d41bc354-129a-5804-8e4c-c37616107c6c"
version = "7.8.3"

[[deps.NLsolve]]
deps = ["Distances", "LineSearches", "LinearAlgebra", "NLSolversBase", "Printf", "Reexport"]
git-tree-sha1 = "019f12e9a1a7880459d0173c182e6a99365d7ac1"
uuid = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
version = "4.5.1"

[[deps.NaNMath]]
deps = ["OpenLibm_jll"]
git-tree-sha1 = "cc0a5deefdb12ab3a096f00a6d42133af4560d71"
uuid = "77ba4419-2d1f-58cd-9bb1-8ffee604a2e3"
version = "1.1.2"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.27+1"

[[deps.OpenLibm_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "05823500-19ac-5b8b-9628-191a04bc5112"
version = "0.8.1+2"

[[deps.OpenSpecFun_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl"]
git-tree-sha1 = "1346c9208249809840c91b26703912dff463d335"
uuid = "efe28fd5-8261-553b-a9e1-b2916fc3738e"
version = "0.5.6+0"

[[deps.OrderedCollections]]
git-tree-sha1 = "cc4054e898b852042d7b503313f7ad03de99c3dd"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.8.0"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "PrecompileTools", "UUIDs"]
git-tree-sha1 = "8489905bcdbcfac64d1daa51ca07c0d8f0283821"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.8.1"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "Random", "SHA", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.11.0"
weakdeps = ["REPL"]

    [deps.Pkg.extensions]
    REPLExt = "REPL"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Colors", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "90af5c9238c1b3b25421f1fdfffd1e8fca7a7133"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.20"

    [deps.PlotlyBase.extensions]
    DataFramesExt = "DataFrames"
    DistributionsExt = "Distributions"
    IJuliaExt = "IJulia"
    JSON3Ext = "JSON3"

    [deps.PlotlyBase.weakdeps]
    DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
    Distributions = "31c24e10-a181-5473-b8eb-7969acd0382f"
    IJulia = "7073ff75-c697-5162-941a-fcdaad2a7d2a"
    JSON3 = "0f8b85d8-7281-11e9-16c2-39a750bddbf1"

[[deps.PlotlyKaleido]]
deps = ["Artifacts", "Base64", "JSON", "Kaleido_jll"]
git-tree-sha1 = "ba551e47d7eac212864fdfea3bd07f30202b4a5b"
uuid = "f2990250-8cf9-495f-b13a-cce12b45703c"
version = "2.2.6"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "BaseDirs", "Colors", "Dates", "Downloads", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "Pkg", "PlotlyBase", "Reexport", "TOML"]
git-tree-sha1 = "1ae939782a5ce9a004484eab5416411c7190d3ce"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.4.6"

    [deps.PlutoPlotly.extensions]
    PlotlyKaleidoExt = "PlotlyKaleido"
    UnitfulExt = "Unitful"

    [deps.PlutoPlotly.weakdeps]
    PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
    Unitful = "1986cc42-f94f-5a68-af5c-568840ba703d"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "7e71a55b87222942f0f9337be62e26b1f103d3e4"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.61"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "36d8b4b899628fb92c2749eb488d884a926614d3"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.3"

[[deps.PrecompileTools]]
deps = ["Preferences"]
git-tree-sha1 = "5aa36f7049a63a1528fe8f7c3f2113413ffd4e1f"
uuid = "aea7be01-6a6a-4083-8856-8a6e6704d82a"
version = "1.2.1"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "9306f6085165d270f7e3db02af26a400d580f5c6"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.4.3"

[[deps.PrettyTables]]
deps = ["Crayons", "LaTeXStrings", "Markdown", "PrecompileTools", "Printf", "Reexport", "StringManipulation", "Tables"]
git-tree-sha1 = "1101cd475833706e4d0e7b122218257178f48f34"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "2.4.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"
version = "1.11.0"

[[deps.PtrArrays]]
git-tree-sha1 = "1d36ef11a9aaf1e8b74dacc6a731dd1de8fd493d"
uuid = "43287f4e-b6f4-7ad1-bb20-aadabca52c3d"
version = "1.3.0"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "StyledStrings", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"
version = "1.11.0"

[[deps.Random]]
deps = ["SHA"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"
version = "1.11.0"

[[deps.RecipesBase]]
deps = ["PrecompileTools"]
git-tree-sha1 = "5c3d09cc4f31f5fc6af001c250bf1278733100ff"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.3.4"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "62389eeff14780bfe55195b7204c0d8738436d64"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.1"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "712fb0231ee6f9120e005ccd56297abbc053e7e0"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.4.8"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"
version = "1.11.0"

[[deps.Setfield]]
deps = ["ConstructionBase", "Future", "MacroTools", "StaticArraysCore"]
git-tree-sha1 = "c5391c6ace3bc430ca630251d02ea9687169ca68"
uuid = "efcf1570-3423-57d1-acb7-fd33fddbac46"
version = "1.1.2"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"
version = "1.11.0"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "66e0a8e672a0bdfca2c3f5937efb8538b9ddc085"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.2.1"

[[deps.SparseArrays]]
deps = ["Libdl", "LinearAlgebra", "Random", "Serialization", "SuiteSparse_jll"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
version = "1.11.0"

[[deps.SpecialFunctions]]
deps = ["IrrationalConstants", "LogExpFunctions", "OpenLibm_jll", "OpenSpecFun_jll"]
git-tree-sha1 = "64cca0c26b4f31ba18f13f6c12af7c85f478cfde"
uuid = "276daf66-3868-5448-9aa4-cd146d93841b"
version = "2.5.0"

    [deps.SpecialFunctions.extensions]
    SpecialFunctionsChainRulesCoreExt = "ChainRulesCore"

    [deps.SpecialFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"

[[deps.StaticArraysCore]]
git-tree-sha1 = "192954ef1208c7019899fbf8049e717f92959682"
uuid = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
version = "1.4.3"

[[deps.Statistics]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "ae3bb1eb3bba077cd276bc5cfc337cc65c3075c0"
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.11.1"
weakdeps = ["SparseArrays"]

    [deps.Statistics.extensions]
    SparseArraysExt = ["SparseArrays"]

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1ff449ad350c9c4cbc756624d6f8a8c3ef56d3ed"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.7.0"

[[deps.StatsBase]]
deps = ["AliasTables", "DataAPI", "DataStructures", "LinearAlgebra", "LogExpFunctions", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "29321314c920c26684834965ec2ce0dacc9cf8e5"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.34.4"

[[deps.StringManipulation]]
deps = ["PrecompileTools"]
git-tree-sha1 = "725421ae8e530ec29bcbdddbe91ff8053421d023"
uuid = "892a3eda-7b42-436c-8928-eab12a02cf0e"
version = "0.4.1"

[[deps.StyledStrings]]
uuid = "f489334b-da3d-4c2e-b8f0-e476e12c162b"
version = "1.11.0"

[[deps.SuiteSparse_jll]]
deps = ["Artifacts", "Libdl", "libblastrampoline_jll"]
uuid = "bea87d4a-7f5b-5778-9afe-8cc45184846c"
version = "7.7.0+0"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "OrderedCollections", "TableTraits"]
git-tree-sha1 = "598cd7c1f68d1e205689b1c2fe65a9f85846f297"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.12.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"
version = "1.11.0"

[[deps.TimeSeries]]
deps = ["Dates", "DelimitedFiles", "DocStringExtensions", "RecipesBase", "Reexport", "Statistics", "Tables"]
git-tree-sha1 = "8b9288d84da88ea44693ca8cf9c236da1778f274"
uuid = "9e3dc215-6440-5c97-bce1-76c03772f85e"
version = "0.23.2"

[[deps.TranscodingStreams]]
git-tree-sha1 = "0c45878dcfdcfa8480052b6ab162cdd138781742"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.11.3"

[[deps.Tricks]]
git-tree-sha1 = "6cae795a5a9313bbb4f60683f7263318fc7d1505"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.10"

[[deps.URIs]]
git-tree-sha1 = "67db6cc7b3821e19ebe75791a9dd19c9b1188f2b"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.5.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"
version = "1.11.0"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"
version = "1.11.0"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+1"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.11.0+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.59.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+2"
"""

# ╔═╡ Cell order:
# ╟─576d5509-527d-4c48-9f2e-007fc977a0d8
# ╟─08782891-e0f5-4c25-9657-24212101724c
# ╟─67198764-1378-4286-8779-8a05011bf084
# ╟─9c11028c-8ccd-4c17-a964-86f09f785ba0
# ╟─e8f80d8c-5b6d-449a-8151-7c6f057492ae
# ╠═92c861a8-3027-4f39-a310-986aa47eea8f
# ╟─c76ef346-c56a-4d0d-a73b-8b81c3688cc4
# ╟─637413e8-c2d2-46d9-ac9c-9a57779bcc33
# ╟─28eff862-eb6a-4546-bfc7-f334aced0e5e
# ╠═11943aea-98c2-4dba-aa08-ab4241a06ffc
# ╟─51a0bef0-7b98-4513-9b83-63b1c944d9a8
# ╠═5e99d909-c2f3-4880-9855-edc3f791370e
# ╠═93a211cf-5ad7-4fdc-a2fb-6981e61a0e3a
# ╠═c7707bee-609b-4a8c-b154-747a279e8e21
# ╟─93580fd5-721b-4183-9e10-2d512eb9e772
# ╟─6f3bab31-f65f-4ca2-9e45-4cc3f630c608
# ╠═0f04c187-bc39-40dc-8832-d5d0f8b62120
# ╠═f1a83428-3842-4eb4-94b7-82c4a5f58966
# ╠═809d9a65-60f8-4281-9d3a-52ba40e913b8
# ╠═86c9852d-e35b-487e-b12a-3576591497ad
# ╠═37b8be8f-9327-462a-974a-2c4bf5328880
# ╟─9181790e-4783-48c6-be76-b2e789d9fc9a
# ╟─b516b76d-9c31-4229-9757-60d0e65b33a0
# ╠═9147a10d-4cd5-470a-b82e-1eb225370008
# ╟─793a93dc-c9d7-48ec-bc64-a4a67e710158
# ╟─d3296808-90cc-4c14-86ac-12718d1d9973
# ╟─bd2321e2-70f5-49ba-9b1f-fca9b3ddbcbd
# ╠═d747423d-e45e-4b03-b604-f426b0ca64e4
# ╠═da6cd1a0-29da-431a-aeb1-f19d161e2538
# ╠═15b7e047-7d56-4dcb-a68f-1794f582dfbd
# ╟─9f4c0a5a-92dc-46bd-9f6a-d980d210cb3f
# ╟─c7a28f02-6e49-4894-9955-66d50de043e3
# ╠═3ceba09c-de39-4411-8efc-d16a765f22fc
# ╟─48d1a9d6-ef89-463a-a768-c56420972d3e
# ╠═8c4a9a0b-e34f-44e7-80d6-e3b176234f2b
# ╟─69c1d54d-9773-4b2e-894b-70de101d1926
# ╠═876f155d-bd0b-4c48-aa81-8c912e21ae0b
# ╟─ce14ef7e-de0b-4032-9631-78e077485f6b
# ╠═fc26b000-601d-426f-a775-8d3663155238
# ╠═d50be6bb-4948-4717-8f90-c47be873b85e
# ╟─8f2eb52e-539c-431e-a2be-d99b4da7e9a0
# ╠═0e52a463-ef5c-41ca-a8dc-900b27370f5f
# ╠═a2f26c7f-6cad-46e5-b4d4-f2b20ce71e4f
# ╟─28f2c903-a27b-4ac8-8cb4-c9091776019b
# ╟─7b6c40ab-c813-4e26-9f81-d2664fcd379c
# ╠═b5c30c98-5757-4ad9-a4c6-129b0b69e88c
# ╟─6b1b2d07-c4e9-480e-809d-fa34a8325541
# ╟─760790e0-f3d5-40c4-b2ee-ca1b8504051d
# ╟─db03fe06-4a63-4a31-9cf3-73969283bcb7
# ╟─60d70eca-eaaf-436d-8096-89e0e5fa778e
# ╟─bc52abd7-d184-4adb-8aa5-c51d8099b8c9
# ╠═0f3fc6f9-f2d3-48a2-a0fb-4f8bcfddb84e
# ╟─94cfd9cf-586d-4a7a-b5ee-a0cb4c3ee288
# ╟─8c4238a8-2ca1-4a88-aee1-95c2afafb871
# ╟─88e1d017-2272-41a0-b97c-10ffc1c7f55b
# ╠═17e30d59-0856-42e1-bf47-20e6d0e37057
# ╠═9ad7d089-0885-4087-9ee0-4f9c35325441
# ╟─32ed885e-cf2f-4cee-b8df-cc81a463d429
# ╟─144c6ac4-4159-4520-bddd-3b75d624a426
# ╟─7bb3d405-e175-4865-a0a0-856a69aeb145
# ╟─23786f58-0f82-4a23-914c-0bb40c63f4e7
# ╟─5107f7cd-d9da-472a-bcba-9f9dd04107f7
# ╟─c1343361-86dd-447a-8ca1-9a2a506bf78d
# ╠═62c41a14-4cdd-488c-a653-0159f10fef1d
# ╟─4ae19cc8-4a3b-4118-af0a-d179f385916b
# ╠═7d2c6085-fd4b-42c7-a27c-b39f6ddf8290
# ╟─dfd8225d-f7de-4740-86de-69a70227afee
# ╟─12bad102-fae5-4d2d-8676-630c59f61956
# ╟─4101ea53-eefa-4af8-b488-3ed431a1cee8
# ╟─fc0fa874-59f8-49c7-b734-4bc38bb60908
# ╟─319d1622-6458-42a4-8e87-4aae991cab61
# ╠═9aa0ccd4-d986-4e6a-ba7b-5652e6edcc6f
# ╟─b09f96de-e2dc-4306-8547-226ec47372a5
# ╟─85191a8a-d381-4580-8c99-0423452f469d
# ╟─2fd32ff5-13f3-41cc-806e-d7e375a4c4ce
# ╟─243c8313-dfa2-4f96-ad95-2e9c815a15ea
# ╟─8be3d1b0-cf85-48a6-966f-3203741f13d3
# ╟─fd2c9069-9b3e-4b5f-8e1a-75728c43a387
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
